/* relax4.c */

#include "env.h"
#include "relax4.h"

int relax4(struct relax4_csa *csa)
{     static const char func[] = "relax4";
      xassert(csa == csa);
      xerror("%s: sorry, this routine is temporarily disabled due to li"
         "censing problems\n", func);
      /* abort(); */
      return -1;
}

void relax4_inidat(struct relax4_csa *csa)
{     static const char func[] = "relax4_inidat";
      xassert(csa == csa);
      xerror("%s: sorry, this routine is temporarily disabled due to li"
         "censing problems\n", func);
      /* abort(); */
}

/* eof */
