#ifndef PRPACK_PREPROCESSED_GRAPH
#define PRPACK_PREPROCESSED_GRAPH

namespace prpack {

    // TODO: this class should not be seeable by the users of the library.
    // Super graph class.
    class prpack_preprocessed_graph {
        public:
            int num_vs;
            int num_es;
            double* d;
    };

}

#endif
